/********************************************************************
 *  Copyright (c) 2003, InfiniTec IT Soluions						*
 *																	*
 *	This program is free software; you can redistribute it			*
 *	and/or modify it under the terms of the GNU General Public		*
 *	License as published by the Free Software Foundation;			*
 *	either version 2 of the License, or (at your option) any		*
 *	later version. This program is distributed in the hope that		*
 *	it will be useful, but WITHOUT ANY WARRANTY; without even the	*	
 *	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR *
 *	PURPOSE. See the GNU General Public License for more details.   * 
 ********************************************************************/
using System;
using System.Net;
using System.Net.Sockets;
using System.Management;

namespace InfiniTec.SystemUtilities
{
	/// <summary>
	/// Provides methods to send so called magic packets to 
	/// a subnet. 
	/// </summary>
	/// <remarks>
	/// A magic packet is a special UDP packet, that is recognised
	/// by WAKE-ON-LAN capable network adapters.
	/// Sending such a packet to a WOL computer will boot it.
	/// </remarks>
	public class MagicPacket
	{
		/// <summary>
		/// Sends a magic packet to the current subnet.
		/// </summary>
		/// <param name="macAddress">The MAC-Address of the computer to start.</param>
		public static void Send(byte[] macAddress) 
		{
			Send(IPAddress.Broadcast, macAddress);
		}

		/// <summary>
		/// Sends a magic packet to the specified subnet.
		/// </summary>
		/// <param name="networkAddress">The MAC-Address of the computer to start</param>
		/// <param name="macAddress">The subnet to send the magic packet to.</param>
		public static void Send(IPAddress networkAddress, byte[] macAddress) 
		{
			UdpClient udpClient;
			byte[] data;
			int i, j;

			data = new byte[114];

			for (i=0; i<6; i++) data[i] = 255;

			for (i=0; i<6; i++) data[i+6] = macAddress[i];

			for (i=0; i<6; i++) data[i+12] = 255;

			for (j=0; j<16; j++) 
			{
				for (i=0; i<6; i++) data[18+(j*6)+i] = macAddress[i];
			}

			udpClient = new UdpClient();
			udpClient.Send(data, 114, new IPEndPoint(networkAddress, 65535));
		}
	}
}
